from __future__ import annotations

from typing import Any, Dict, Optional
from fastmcp import FastMCP
import json

from .config import get_settings
from .embeddings import Embeddings
from .qdr_client import QdrClient


mcp = FastMCP("qdrant-mcp-python")
_qdr = QdrClient()
_settings = get_settings()


@mcp.tool(
    name="memory-store",
    description=(
        "Store long-term textual information in the underlying vector memory store. "
        "Automatically embeds the text with OpenAI and returns the stored ID."
    ),
)
def memory_store(
    information: str,
    metadata: Optional[Dict[str, Any]] = None,
    collection_name: Optional[str] = None,
) -> str:
    """Store text into Qdrant with dense vectors produced by OpenAI embeddings.

    information: The main text content to store as long-term memory
    metadata: Optional JSON metadata to attach
    collection_name: Optional collection to use; defaults to env COLLECTION_NAME
    """
    collection = collection_name or _settings.default_collection
    if not collection:
        raise ValueError("Collection name is required")

    vector = Embeddings.embed_one(information)

    # Ensure collection exists with the right dimensionality
    _qdr.ensure_collection(collection, len(vector))

    point_id = str(__import__("time").time_ns())
    payload: Dict[str, Any] = {
        "information": information,
        "stored_at": __import__("datetime").datetime.utcnow().isoformat() + "Z",
    }
    if metadata:
        payload["metadata"] = metadata

    _qdr.upsert_points(
        collection,
        [
            {
                "id": point_id,
                "vector": (
                    {"dense": vector}
                    if (
                        isinstance(
                            _qdr.collection_info(collection)
                            .get("config", {})
                            .get("params", {})
                            .get("vectors"),
                            dict,
                        )
                        and "dense"
                        in _qdr.collection_info(collection)
                        .get("config", {})
                        .get("params", {})
                        .get("vectors", {})
                    )
                    else vector
                ),
                "payload": payload,
            }
        ],
    )

    return f"Information stored successfully in collection '{collection}' with ID: {point_id}"


@mcp.tool(
    name="memory-search",
    description=(
        "Retrieve relevant information previously stored in vector memory using semantic (dense) search with OpenAI embeddings."
    ),
)
def memory_search(
    query: str,
    limit: int = 5,
    collection_name: Optional[str] = None,
) -> str:
    """Search for similar items in Qdrant using OpenAI embeddings for the query."""
    collection = collection_name or _settings.default_collection
    if not collection:
        raise ValueError("Collection name is required")

    query_vec = Embeddings.embed_one(query)
    # Only pass a vector name if the collection defines a named 'dense' vector
    try:
        _info = _qdr.collection_info(collection)
        _vectors = (_info or {}).get("config", {}).get("params", {}).get("vectors")
        use_dense = isinstance(_vectors, dict) and "dense" in _vectors
    except Exception:
        use_dense = False

    results = _qdr.search(
        collection,
        query_vec,
        limit,
        vector_name=("dense" if use_dense else None),
    )

    if not results:
        return f'No relevant information found for query: "{query}"'

    # Return only structured data as serialized text (JSON string)
    structured_results: list[Dict[str, Any]] = []
    for r in results:
        structured_results.append(
            {
                "score": r.get("score", 0.0),
                "id": r.get("id"),
                "payload": r.get("payload", {}),
            }
        )

    return json.dumps(structured_results, ensure_ascii=False)


@mcp.tool(
    name="memory-debug",
    description=(
        "Debug/inspection tool for memory collections. Shows collection info and sample payloads."
    ),
)
def memory_debug(
    collection_name: Optional[str] = None,
) -> str:
    collection = collection_name or _settings.default_collection
    if not collection:
        raise ValueError("Collection name is required")

    info = _qdr.collection_info(collection)
    samples = _qdr.scroll_samples(collection, limit=5)

    out = [
        f'Collection Info for "{collection}":',
        __import__("json").dumps(info, indent=2),
    ]
    out.append("")
    out.append(f"Sample Data (first {len(samples)} points):")
    for idx, p in enumerate(samples, start=1):
        out.append(f"\n--- Point {idx} (ID: {p.get('id')}) ---")
        payload = p.get("payload", {})
        out.append(f"Payload keys: {', '.join(payload.keys())}")
        out.append("Payload: " + __import__("json").dumps(payload, indent=2))

    return "\n".join(out)


def run() -> None:
    # Default transport is stdio in fastmcp; be explicit
    mcp.run(transport="stdio")



from __future__ import annotations

from typing import Any, Dict, List, Optional
from urllib.parse import urlparse
from qdrant_client import QdrantClient
from qdrant_client.http import models as qm
from .config import get_settings


class QdrClient:
    def __init__(self) -> None:
        s = get_settings()
        parsed = urlparse(s.qdrant_url)
        if parsed.port is not None:
            port = parsed.port
        elif parsed.scheme == "http":
            port = 80
        else:
            port = 443
        self._client = QdrantClient(
            prefer_grpc=False,
            port=port,
            url=s.qdrant_url,
            api_key=s.qdrant_api_key,
            timeout=s.qdrant_timeout,
        )

    def ensure_collection(self, name: str, vector_size: int) -> None:
        try:
            self._client.create_collection(
                collection_name=name,
                # Create as a named vector collection with 'dense' vector
                vectors_config={
                    "dense": qm.VectorParams(
                        size=vector_size, distance=qm.Distance.COSINE
                    )
                },
            )
        except Exception as e:  # collection may already exist
            # check exists; if exists, ignore, else re-raise
            try:
                info = self._client.get_collection(name)
                if info is None:
                    raise
            except Exception:
                raise e

    def upsert_points(
        self,
        name: str,
        points: List[Dict[str, Any]],
    ) -> None:
        # points: [{id, vector, payload}]
        self._client.upsert(
            collection_name=name,
            points=[
                qm.PointStruct(id=p["id"], vector=p["vector"], payload=p.get("payload"))
                for p in points
            ],
        )

    def search(
        self,
        name: str,
        vector: List[float],
        limit: int = 5,
        vector_name: Optional[str] = None,
    ) -> List[Dict[str, Any]]:
        results = self._client.search(
            collection_name=name,
            query_vector=vector
            if vector_name is None
            else qm.NamedVector(name=vector_name, vector=vector),
            limit=limit,
            with_payload=True,
        )
        out: List[Dict[str, Any]] = []
        for r in results:
            out.append(
                {
                    "id": r.id,
                    "score": r.score,
                    "payload": r.payload,
                }
            )
        return out

    def scroll_samples(self, name: str, limit: int = 5) -> List[Dict[str, Any]]:
        points, _ = self._client.scroll(
            collection_name=name,
            with_payload=True,
            with_vectors=False,
            limit=limit,
        )
        return [{"id": p.id, "payload": p.payload} for p in points]

    def collection_info(self, name: str) -> Dict[str, Any]:
        info = self._client.get_collection(name)
        # qdrant-client returns a typed object; convert to dict
        return info.dict() if hasattr(info, "dict") else dict(info)  # type: ignore[arg-type]



import os
from dataclasses import dataclass


@dataclass(frozen=True)
class Settings:
    qdrant_url: str = os.getenv("QDRANT_URL", "http://localhost:6333")
    qdrant_api_key: str | None = os.getenv("QDRANT_API_KEY")
    default_collection: str | None = os.getenv("COLLECTION_NAME")
    qdrant_timeout: float = float(os.getenv("QDRANT_TIMEOUT", "30"))
    # OpenAI embeddings
    openai_api_key: str | None = os.getenv("OPENAPI_API_KEY") or os.getenv(
        "OPENAI_API_KEY"
    )
    openai_base_url: str | None = os.getenv("OPENAI_BASE_URL")
    openai_embedding_model: str = os.getenv(
        "OPENAI_EMBEDDING_MODEL", "text-embedding-3-small"
    )
    openai_timeout: float = float(os.getenv("OPENAI_TIMEOUT", "30"))
    # Preferred vector name for multi-vector collections
    default_vector_name: str = os.getenv("DEFAULT_VECTOR_NAME", "dense")


def get_settings() -> Settings:
    return Settings()



from __future__ import annotations

from typing import Iterable, List
from openai import OpenAI
from .config import get_settings


class Embeddings:
    _client: OpenAI | None = None

    @classmethod
    def client(cls) -> OpenAI:
        if cls._client is None:
            s = get_settings()
            if not s.openai_api_key:
                raise RuntimeError(
                    "OpenAI API key is required (OPENAPI_API_KEY or OPENAI_API_KEY)"
                )
            cls._client = OpenAI(
                api_key=s.openai_api_key,
                base_url=s.openai_base_url,
                timeout=s.openai_timeout,
                max_retries=2,
            )
        return cls._client

    @classmethod
    def embed_one(cls, text: str) -> List[float]:
        s = get_settings()
        resp = cls.client().embeddings.create(
            model=s.openai_embedding_model,
            input=text,
        )
        return list(resp.data[0].embedding)

    @classmethod
    def embed_many(cls, texts: Iterable[str]) -> List[List[float]]:
        s = get_settings()
        # OpenAI API supports batching, but to keep behavior simple, call once with list
        resp = cls.client().embeddings.create(
            model=s.openai_embedding_model,
            input=list(texts),
        )
        return [list(item.embedding) for item in resp.data]
